(* portlandViewControllerD.ml     View controller delegate for Portland
 *
 * Copyright (c) 2016 Psellos   http://psellos.com
 * Licensed under the MIT license:
 *     http://www.opensource.org/licenses/mit-license.php
 *)

class t =
object (self)
    inherit Wrappee.t

    (* Don't bother updating the display when the app is in the
     * background.
     *)
    val mutable active: bool = true
    val mutable vwd: float = 0.0    (* Screen width *)
    val mutable vht: float = 0.0    (* Screen height *)

    val mutable theLabel: UiLabel.t option = None

    method label =
        match theLabel with
        | None -> raise Not_found  (* Doesn't happen *)
        | Some label -> label

    method setLabel' aLabel =
        theLabel <- Some aLabel

    method activate =
        active <- true

    method deactivate =
        active <- false

    method viewController'viewWillAppear'
            (vc: Ui.portlandViewController) (animated: bool) =
        let app = UiApplication.sharedApplication () in
        let (_, _, w, h) = app#delegate#window#frame in
        vwd <- w;
        vht <- h

    method viewController'viewWillDisappear'
            (vc: Ui.portlandViewController) (animated: bool) =
        ()

    method viewControllerShouldAutorotate' (vc: Ui.portlandViewController) =
        false

    method timerTick' (timer: NsTimer.t) =
        self#display

    method private display =
        if active then
            let (angle, width, height, name) =
                match (UiDevice.currentDevice())#orientation with
                | UiDevice.OrientationUnknown ->
                    (0.0, vwd, vht, "Unknown")
                | UiDevice.OrientationPortrait ->
                    (0.0, vwd, vht, "Portrait")
                | UiDevice.OrientationPortraitUpsideDown ->
                    (3.1415926535897932, vwd, vht, "Portrait Upside Down")
                | UiDevice.OrientationLandscapeLeft ->
                    (1.5707963267948966, vht, vwd, "Landscape Left")
                | UiDevice.OrientationLandscapeRight ->
                    (-1.5707963267948966, vht, vwd, "Landscape Right")
                | UiDevice.OrientationFaceUp ->
                    (0.0, vwd, vht, "Face Up")
                | UiDevice.OrientationFaceDown ->
                    (0.0, vwd, vht, "Face Down")
            in
            match theLabel with
            | None -> ()
            | Some label ->
                label#setText' (Printf.sprintf "\xce\xa8\n%s" name);
                label#setTransform' (CgAffineTransform.makeRotation angle);
                label#setBounds' (0.0, 0.0, width, height)
end

let _ =
    let wrapped robjcv =
        let t = new t in let () = t#setContainer robjcv in t
    in
    Callback.register "PortlandViewControllerD.wrapped" wrapped
